# -*- coding: utf-8 -*-

from acentoweb.collectionactions import _
from plone import schema
from plone.autoform.interfaces import IFormFieldProvider
from plone.dexterity.interfaces import IDexterityContent
from plone.supermodel import model
from plone.autoform import directives
from zope.component import adapter
from zope.interface import Interface
from zope.interface import implementer
from zope.interface import provider
from z3c.relationfield.schema import RelationChoice
from z3c.relationfield.schema import RelationList
from plone.app.z3cform.widget import RelatedItemsFieldWidget
from plone.app.vocabularies.catalog import CatalogSource


class ICollectionCopyMarker(Interface):
    pass

@provider(IFormFieldProvider)
class ICollectionCopy(model.Schema):
    """
    """

    linked_copy_folder =  RelationChoice(
         title=_(u'Linked Copy folder'),
         description=_(u'Folder to copy items to'),
         required=False,
         vocabulary='plone.app.vocabularies.Catalog',
    )

    directives.widget(
          'linked_copy_folder',
          RelatedItemsFieldWidget,
          pattern_options={
              'selectableTypes': ['Folder'],
              #pattern_options=make_relation_root_path,
          },
     )



@implementer(ICollectionCopy)
@adapter(ICollectionCopyMarker)
class CollectionCopy(object):
    def __init__(self, context):
        self.context = context

    @property
    def linked_copy_folder(self):
        if hasattr(self.context, 'linked_copy_folder'):
            return self.context.linked_copy_folder
        return None

    @linked_copy_folder.setter
    def linked_copy_folder(self, value):
        self.context.linked_copy_folder = value



# -*- coding: utf-8 -*-

from AccessControl import getSecurityManager
from Acquisition import aq_inner
from Acquisition import aq_parent
from Products.Five.browser import BrowserView
from Products.statusmessages.interfaces import IStatusMessage
from zExceptions import Unauthorized
from zope.event import notify
from zope.interface import Interface
import transaction


from plone import api

#Potential needed imports
#import six
#from OFS.CopySupport import CopyError
#from Products.CMFCore.utils import getToolByName
#from Products.CMFPlone import PloneMessageFactory as _
#from Products.CMFPlone.utils import safe_unicode
#from ZODB.POSException import ConflictError
#from z3c.form import button
#from z3c.form import field
#from z3c.form import form
#from z3c.form.widget import ComputedWidgetAttribute
#from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
#from zope.lifecycleevent import ObjectModifiedEvent
#from zope import schema
#from zope.component import getMultiAdapter
#from zope.component import queryMultiAdapter
#from zope.container.interfaces import INameChooser

class CollectionMove(BrowserView):

    def __call__(self):
        messages = IStatusMessage(self.request)
        context = self.context

        #Get folder we move the object  to
        to_folder = context.linked_folder.to_object

        ## TO DO: Check if any item is locked ??
        #plone.api.content.move(source=None, target=None, id=None, safe_id=False)[source]¶
        #Move the object to the target container.

        #Parameters
        #source (Content object) – [required] Object that we want to move.
        #target (Folderish content object) – Target container to which the source object will be moved. If no target is specified, the source object’s container will be used as a target, effectively making this operation a rename (Rename content).
        #id (string) – Pass this parameter if you want to change the id of the moved object on the target location. If the new id conflicts with another object in the target container, a suffix will be added to the moved object’s id.
        #safe_id (boolean) – When False, the given id will be enforced. If the id is conflicting with another object in the target container, raise a InvalidParameterError. When True, choose a new, non-conflicting id.

        #Returns
        #Content object that was moved to the target location

        #Raises
        #KeyError ValueError

        #portal = api.portal.get()

        #except KeyError:
        #    messages.add(u"Folder does not exist", type="warning")

        if to_folder:

            for item in  context.restrictedTraverse('@@contentlisting')():
                try:
                    api.content.move(source=item.getObject(), target=to_folder)
                    messages.add('Moved item: ' + item.id + ' to folder: ' + to_folder.Title(), type="info")
                except KeyError:
                    messages.add(u"Folder does not exist", type="warning")

        else:
            messages.add(u'Nothing to move', type="info")

        self.request.response.redirect(context.absolute_url())



class CollectionCopy(BrowserView):

    def __call__(self):
        messages = IStatusMessage(self.request)
        context = self.context

        to_folder = context.linked_folder.to_object

        if to_folder:
            for item in  context.restrictedTraverse('@@contentlisting')():
                try:
                    api.content.copy(source=item.getObject(), target=to_folder)
                    messages.add('Copied item: ' + item.id + ' to folder: ' + to_folder.Title(), type="info")
                except KeyError:
                    messages.add(u"Folder does not exist", type="warning")

        else:
            messages.add(u'Nothing to copy', type="info")

        self.request.response.redirect(context.absolute_url())



# -*- coding: utf-8 -*-
from acentoweb.collectionactions.behaviors.collection_move import ICollectionMoveMarker
from acentoweb.collectionactions.testing import ACENTOWEB_COLLECTIONACTIONS_INTEGRATION_TESTING  # noqa
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.behavior.interfaces import IBehavior
from zope.component import getUtility

import unittest


class CollectionMoveIntegrationTest(unittest.TestCase):

    layer = ACENTOWEB_COLLECTIONACTIONS_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        setRoles(self.portal, TEST_USER_ID, ['Manager'])

    def test_behavior_collection_move(self):
        behavior = getUtility(IBehavior, 'acentoweb.collectionactions.collection_move')
        self.assertEqual(
            behavior.marker,
            ICollectionMoveMarker,
        )
